## Copyright (c) 2005-2006, Gillmer J. Derge.

## This file is part of Civilization IV Great Options
##
## Civilization IV Great Options is free software; you can redistribute
## it and/or modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation; either
## version 2 of the License, or (at your option) any later version.
##
## Civilization IV Great Options is distributed in the hope that it will
## be useful, but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
## See the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Civilization IV Great Options; if not, write to the Free
## Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
## 02110-1301 USA
	
from CvPythonExtensions import *
import CvUtil
import CvEventManager
import CvGreatGeneralEventManager
import CvGreatStatesmanEventManager
import CvGDEventManager
import CvFieldHospitalEventManager
import CvGreatOptionsEventManager
import CvGPTrickleEventManager
import CvUnitAllegianceEventManager
import CvRandomGreatPersonBirthsEventManager
import CvMADNukesEventManager
import CvNoNameGreatPersonRenamerModEventManager

import CvMilitaryBasesEventManager
import CvSniperEventManager
import CvMercEventManager
import CvCulturalInfluencesEventManager
import CvCulturalDecayEventManager
import CvEnhancedCultureConquestEventManager
import CvEnhancedTechConquestEventManager
import CvLocalizedStartingTechsEventManager
import CvImmigrationEventManager

import CvConfigParser

gc = CyGlobalContext()


# Change the value to enable or disable the features from the GP Trickle Mod.
# Default value is true
g_bGPTrickleFeaturesEnabled = true

# Change the value to enable or disable the features from the Unit Allegiance 
# Mod.
# Default value is true
g_bUnitAllegianceFeaturesEnabled = true

# Change the value to enable or disable the features from the Random Great 
# Person Births Mod.
# Default value is true
g_bRandomGreatPersonBirthFeaturesEnabled = true

# Change the value to enable or disable the features from the M.A.D. Nukes Mod.
# Default value is true
g_bMADNukesFeaturesEnabled = true

# Change the value to enable or disable the features from the No Name Great 
# Person Renamer Mod.
# Default value is true
g_bNoNameGreatPersonRenamerFeaturesEnabled = true

# Change the value to enable or disable the features from the Cultural 
# Influences mod.
# Default value is true
g_bCulturalInfluencesFeaturesEnabled = true

# Change the value to enable or disable the features from the Mercenaries Mod.
# Default value is true
g_bMercenariesFeaturesEnabled = true

# Change the value to enable or disable the features from the Enhanced Culture 
# Conquest mod.
# Default value is true
g_bEnhancedCultureConquestFeaturesEnabled = true

# Change the value to enable or disable the features from the Enhanced Tech 
# Conquest mod.
# Default value is true
g_bEnhancedTechConquestFeaturesEnabled = true

# Change the value to enable or disable the features from the Localized 
# Starting Techs Mod.
# Default value is true
g_bLocalizedStartingTechsFeaturesEnabled = true

# Change the value to enable or disable the features from the Immigration Mod.
# Default value is true
g_bImmigrationModFeaturesEnabled = true


def loadConfigurationData():
	
	CvUtil.pyPrint("Loading configuration values")
	
	global g_bGPTrickleModEnabled
	global g_bUnitAllegianceFeaturesEnabled
	global g_bRandomGreatPersonBirthFeaturesEnabled
	global g_bMADNukesFeaturesEnabled
	global g_bNoNameGreatPersonRenamerFeaturesEnabled
	global g_bCulturalInfluencesFeaturesEnabled
	global g_bMercenariesFeaturesEnabled
	global g_bEnhancedCultureConquestFeaturesEnabled 
	global g_bEnhancedTechConquestFeaturesEnabled
	global g_bLocalizedStartingTechsFeaturesEnabled
	global g_bImmigrationModFeaturesEnabled

	config = CvConfigParser.CvConfigParser("Great Options Mod Config.ini")

	if(config != None):
		g_bGPTrickleFeaturesEnabled = config.getboolean("Great Options Mod", "GP Trickle Features Enabled", True)
		g_bUnitAllegianceFeaturesEnabled = config.getboolean("Great Options Mod", "Unit Allegiance Features Enabled", True)
		g_bRandomGreatPersonBirthFeaturesEnabled = config.getboolean("Great Options Mod", "Random Great Person Birth Features Enabled", True)
		g_bMADNukesFeaturesEnabled = config.getboolean("Great Options Mod", "MAD Nukes Features Enabled", True)
		g_bNoNameGreatPersonRenamerFeaturesEnabled = config.getboolean("Great Options Mod", "No Name Great Person Renamer Features Enabled", True)
		g_bCulturalInfluencesFeaturesEnabled = config.getboolean("Great Options Mod", "Cultural Influences Features Enabled", True)
		g_bMercenariesFeaturesEnabled = config.getboolean("Great Options Mod", "Mercenaries Features Enabled", True)		
		g_bEnhancedCultureConquestFeaturesEnabled = config.getboolean("Great Options Mod", "Enhanced Culture Conquest Features Enabled", True)
		g_bEnhancedTechConquestFeaturesEnabled = config.getboolean("Great Options Mod", "Enhanced Tech Conquest Features Enabled", True)
		g_bLocalizedStartingTechsFeaturesEnabled = config.getboolean("Great Options Mod", "Localized Starting Techs Features Enabled", True)
		g_bImmigrationModFeaturesEnabled = config.getboolean("Great Options Mod", "ImmigrationModFeatures Enabled", True)


class CvCustomEventManager(CvEventManager.CvEventManager, object):

	CustomEvents = {}
	EventManagers = {}
		
	def __init__(self, *args, **kwargs):

		super(CvCustomEventManager, self).__init__(*args, **kwargs)
		# map the initial EventHandlerMap values into the new data structure
		for eventType, eventHandler in self.EventHandlerMap.iteritems():
			self.setEventHandler(eventType, eventHandler)
		
		# --> INSERT EVENT HANDLER INITIALIZATION HERE <--
		CvGreatGeneralEventManager.CvGreatGeneralEventManager(self)
		CvGreatStatesmanEventManager.CvGreatStatesmanEventManager(self)
		CvGDEventManager.CvGDEventManager(self)
		CvFieldHospitalEventManager.CvFieldHospitalEventManager(self)
		
		if(g_bGPTrickleFeaturesEnabled):
			CvGPTrickleEventManager.CvGPTrickleEventManager(self)

		if(g_bUnitAllegianceFeaturesEnabled):	        
			CvUnitAllegianceEventManager.CvUnitAllegianceEventManager(self)

		if(g_bRandomGreatPersonBirthFeaturesEnabled):
			CvRandomGreatPersonBirthsEventManager.CvRandomGreatPersonBirthsEventManager(self)
		
		if(g_bMADNukesFeaturesEnabled):
			CvMADNukesEventManager.CvMADNukesEventManager(self)
		
		if(g_bNoNameGreatPersonRenamerFeaturesEnabled):
			CvNoNameGreatPersonRenamerModEventManager.CvNoNameGreatPersonRenamerModEventManager(self)
			
		if(g_bCulturalInfluencesFeaturesEnabled):
			CvCulturalInfluencesEventManager.CvCulturalInfluencesEventManager(self)
			CvCulturalDecayEventManager.CvCulturalDecayEventManager(self)

		if(g_bMercenariesFeaturesEnabled):
			CvMercEventManager.CvMercEventManager(self)
	
		if(g_bEnhancedCultureConquestFeaturesEnabled):
			CvEnhancedCultureConquestEventManager.CvEnhancedCultureConquestEventManager(self)

		if(g_bEnhancedTechConquestFeaturesEnabled):
			CvEnhancedTechConquestEventManager.CvEnhancedTechConquestEventManager(self)

		CvMilitaryBasesEventManager.CvMilitaryBasesEventManager(self)

		self.EventManagers["CvSniperEventManager"] = CvSniperEventManager.CvSniperEventManager(self)

		if(g_bLocalizedStartingTechsFeaturesEnabled):	
			CvLocalizedStartingTechsEventManager.CvLocalizedStartingTechsEventManager(self)
		
		if(g_bImmigrationModFeaturesEnabled):
			CvImmigrationEventManager.CvImmigrationEventManager(self)

		CvGreatOptionsEventManager.CvGreatOptionsEventManager(self)


	def getEventManager(self, strEventManagerName):
		
		return self.EventManagers[strEventManagerName]


	def beginEvent( self, context, argsList=-1 ):
		"Begin Event"
		if(self.CustomEvents.has_key(context)):
			return self.CustomEvents[context][2](argsList)
		else:
			return CvEventManager.CvEventManager.beginEvent(self, context, argsList)
			
	def applyEvent( self, argsList ):
		'Apply the effects of an event '
		context, playerID, netUserData, popupReturn = argsList
		
		if(self.CustomEvents.has_key(context)):
			entry = self.CustomEvents[context]
			# the apply function
			return entry[1]( playerID, netUserData, popupReturn )   
		else:
			return CvEventManager.CvEventManager.applyEvent(self, argsList)


	def addCustomEventDefinition(self, eventType, eventDefinition):
		self.CustomEvents[eventType] = eventDefinition

	def removeCustomEventDefinition(self, eventType):
		del self.CustomEvents[eventType]

	def setCustomEventDefinition(self, eventType, eventDefinition):
		self.CustomEvents[eventType] = eventDefinition

	def addEventHandler(self, eventType, eventHandler, bBeforeParent=False):
		"""Adds a handler for the given event type.
		
		A list of supported event types can be found in the initialization 
		of EventHandlerMap in the CvEventManager class.

		"""
		if(bBeforeParent):
			self.EventHandlerMap[eventType].insert(0,eventHandler)
		else:
			self.EventHandlerMap[eventType].append(eventHandler)

	def removeEventHandler(self, eventType, eventHandler):
		"""Removes a handler for the given event type.
		
		A list of supported event types can be found in the initialization 
		of EventHandlerMap in the CvEventManager class.  It is an error if 
		the given handler is not found in the list of installed handlers.

		"""
		self.EventHandlerMap[eventType].remove(eventHandler)
	
	def setEventHandler(self, eventType, eventHandler):
		"""Removes all previously installed event handlers for the given 
		event type and installs a new handler .
		
		A list of supported event types can be found in the initialization 
		of EventHandlerMap in the CvEventManager class.  This method is 
		primarily useful for overriding, rather than extending, the default 
		event handler functionality.

		"""
		self.EventHandlerMap[eventType] = [eventHandler]

	def handleEvent(self, argsList):
		"""Handles events by calling all installed handlers."""
		self.origArgsList = argsList
		flagsIndex = len(argsList) - 6
		self.bDbg, self.bMultiPlayer, self.bAlt, self.bCtrl, self.bShift, self.bAllowCheats = argsList[flagsIndex:]		
		eventType = argsList[0]
		return {
			"kbdEvent": self._handleConsumableEvent,
			"mouseEvent": self._handleConsumableEvent,
			"OnSave": self._handleOnSaveEvent,
			"OnLoad": self._handleOnLoadEvent
		}.get(eventType, self._handleDefaultEvent)(eventType, argsList[1:])

	def _handleDefaultEvent(self, eventType, argsList):
		if self.EventHandlerMap.has_key(eventType):
			for eventHandler in self.EventHandlerMap[eventType]:
				# the last 6 arguments are for internal use by handleEvent
				eventHandler(argsList[:len(argsList) - 6])

	def _handleConsumableEvent(self, eventType, argsList):
		"""Handles events that can be consumed by the handlers, such as
		keyboard or mouse events.
		
		If a handler returns non-zero, processing is terminated, and no 
		subsequent handlers are invoked.

		"""
		if self.EventHandlerMap.has_key(eventType):
			for eventHandler in self.EventHandlerMap[eventType]:
				# the last 6 arguments are for internal use by handleEvent
				result = eventHandler(argsList[:len(argsList) - 6])
				if (result > 0):
					return result
		return 0

	# TODO: this probably needs to be more complex
	def _handleOnSaveEvent(self, eventType, argsList):
		"""Handles OnSave events by concatenating the results obtained
		from each handler to form an overall consolidated save string.

		"""
		result = ""
		if self.EventHandlerMap.has_key(eventType):
			for eventHandler in self.EventHandlerMap[eventType]:
				# the last 6 arguments are for internal use by handleEvent
				result = result + eventHandler(argsList[:len(argsList) - 6])
		return result

	# TODO: this probably needs to be more complex
	def _handleOnLoadEvent(self, eventType, argsList):
		"""Handles OnLoad events."""
		return self._handleDefaultEvent(eventType, argsList)
