#
# Military Bases Mod
# MilitaryBasesUtils
# 

from CvPythonExtensions import *
import CvUtil

import sys
import PyHelpers
import CvConfigParser


import pickle

################# SD-UTILITY-PACK ###################
import SdToolKit
sdEcho         = SdToolKit.sdEcho
sdModInit      = SdToolKit.sdModInit
sdModLoad      = SdToolKit.sdModLoad
sdModSave      = SdToolKit.sdModSave
sdEntityInit   = SdToolKit.sdEntityInit
sdEntityExists = SdToolKit.sdEntityExists
sdGetVal       = SdToolKit.sdGetVal
sdSetVal       = SdToolKit.sdSetVal
sdEntityWipe   = SdToolKit.sdEntityWipe

gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo

# Change this value to false if drill sergeants should not be required to train
# units in military bases.
# Default value is true
g_bDrillSergeantsRequiredToTrain = true

# Change this value to increase or decrease the amount of experience units will
# gain from the training given by drill sergeants who have joined a city. If 
# "Drill Sergeants Required To Train" is set to false then this value will 
# stack with the value in "Base Training Experience".
# Default value is 1
g_iDrillSergeantExperienceBonus = 1

# Change this value to increase or decrease the number of units each drill 
# sergeant can train. This value will only be used if drill sergeants are 
# required to train units.
# Default value is 3
g_iUnitsPerDrillSergeant = 3

# Change this value to false if units should get experience point every turn 
# instead of randomly, depending on their current level and experience points.
# Default value it true
g_bUnitsGainExperienceRandomly = true

# Change this value to increase or decrease the amount of experience units gain
# when they are trained successfully.
# Default value is 1
g_iBaseTrainingExperience = 1

# Change this value to false if there should be no max level that units can be
# trained in the military bases.
# Default value is true
g_bMaxLevelUnitTrainedCapped = True

# Change this value to increase or decrease the number of levels that units can
# be trained in the military bases.
# Default value is 5
g_iMaxLevelUnitTrained = 5


# globals
###################################################
class MilitaryBasesUtils:

	def __init__(self):
		# initialize base class

		global g_bDrillSergeantsRequiredToTrain
		global g_iDrillSergeantExperienceBonus
		global g_iUnitsPerDrillSergeant
		global g_bUnitsGainExperienceRandomly
		global g_iBaseTrainingExperience
		global g_bMaxLevelUnitTrainedCapped
		global g_iMaxLevelUnitTrained

		config = CvConfigParser.CvConfigParser("Military Bases Config.ini")
		
		# Read the configuration values for the Military Bases
		if(config != None):
			g_bDrillSergeantsRequiredToTrain = config.getboolean("Military Bases", "Drill Sergeants Required To Train", true)
			g_iDrillSergeantExperienceBonus = config.getint("Military Bases", "Drill Sergeant Experience Bonus", 1)
			g_iUnitsPerDrillSergeant = config.getint("Military Bases", "Units Per Drill Sergeant", 3)
			g_bUnitsGainExperienceRandomly = config.getboolean("Military Bases", "Units Gain Experience Randomly", true)
			g_iBaseTrainingExperience = config.getint("Military Bases", "Base Training Experience", 1)
			g_bMaxLevelUnitTrainedCapped = config.getboolean("Military Bases", "Max Level Unit Trained Capped", true)
			g_iMaxLevelUnitTrained = config.getint("Military Bases", "Max Level Unit Trained", 5)

		
	# Returns true if the unit can be trained, false otherwise.
	def canTrainUnit(self, objUnit):
	
		# Return false immediately if the unit passed in is invalid
		if(objUnit == None):
			return false
			
		# Return false immediately if the unit passed in is invalid	
		if(objUnit.isNone()):
			return false

		# Return false if the unit is past the point of training
		if(g_bMaxLevelUnitTrainedCapped and objUnit.getLevel() >= g_iMaxLevelUnitTrained):
			return false
		
		# Return false if the unit cannot move
		if(not objUnit.canMove()):
			return false
			
		# Get the plot the unit is on
		objPlot = objUnit.plot()
		
		# Return false if the unit passed in is not in a city
		if(not objPlot.isCity()):
			return false
		
		# Return false if the unit is already training
		if(objUnit.getGroup().getActivityType() == ActivityTypes.ACTIVITY_SLEEP):
			return false
			
		# Return false if the unit is ready to be promoted
		if(objUnit.isPromotionReady()):
			return false
			
		# Get the city on the plot
		objCity = objPlot.getPlotCity()

		# Get the number of drill sergeants in the city
		iDrillSergeantCount = objCity.getFreeSpecialistCount(gc.getInfoTypeForString("SPECIALIST_DRILL_SERGEANT"))
		
		# Calculate the maximum number units that can be trained in the city.
		iMaxUnitTrainingCount = g_iUnitsPerDrillSergeant * iDrillSergeantCount
		
		# Return false if the city the unit is in does not belong to the units
		# owner
		if(objCity.getOwner() != objUnit.getOwner()):
			return false
					
		# Return false if drill sergeants are required to train units and the 
		# city the unit is at doesn't have any.
		if(g_bDrillSergeantsRequiredToTrain and iDrillSergeantCount == 0):
			return false
 		
 		# Get the number of units currrently being trained in the city.
 		iTrainingCount = self.getNumCityTrainingUnits(objCity)
 		
 		# Return false if drill sergeants are required to train units and the
 		# number of units being trained in the city has reached the maximum
 		# amount
 		if(g_bDrillSergeantsRequiredToTrain and iTrainingCount >= iMaxUnitTrainingCount):
 			return false
 		
 		# Return false if the unit is a land unit and the city doesn't have an 
 		# army base
		if(not objCity.hasBuilding(gc.getInfoTypeForString("BUILDING_ARMYBASE")) and objUnit.getDomainType() == gc.getInfoTypeForString("DOMAIN_LAND")):
			return false

 		# Return false if the unit is a sea unit and the city doesn't have a 
 		# navy base
		elif(not objCity.hasBuilding(gc.getInfoTypeForString("BUILDING_NAVYBASE")) and objUnit.getDomainType() == gc.getInfoTypeForString("DOMAIN_SEA")):
			return false

 		# Return false if the unit is an air unit and the city doesn't have an 
 		# airforce base
		elif(not objCity.hasBuilding(gc.getInfoTypeForString("BUILDING_AIRFORCEBASE")) and objUnit.getDomainType() == gc.getInfoTypeForString("DOMAIN_AIR")):
			return false
			
		return true


	# Try to train the units in the city
	def trainCityUnits(self, objCity):
	
		# Get the list of units in the city			
		unitList = self.getCityUnitList(objCity)
		
		# Go through each unit in the list and try to train them
		for objUnit in unitList:
			self.trainUnit(objUnit)
			
	
	# Train the unit giving it experience points if needed
	def trainUnit(self, objUnit):

		# Return immediately if the unit passed in is invalid
		if(objUnit == None):
			return
		
		# Return immediately if the unit passed in is invalid
		if(objUnit.isNone()):
			return 

		# Return immediately if the unit isn't being tracked.
		if(sdEntityExists("Military Bases",self.getUniqueUnitID(objUnit)) == False):
			return
		
		# If the unit isn't sleeping but is being tracked, stop tracking the 
		# unit and return immediately
		if(objUnit.getGroup().getActivityType() != ActivityTypes.ACTIVITY_SLEEP):
			sdEntityWipe("Military Bases", self.getUniqueUnitID(objUnit))	
			return
			
		# Return if the unit is ready to be promoted
		if(objUnit.isPromotionReady()):
			objSelectionGroup = objUnit.getGroup()
			objSelectionGroup.setActivityType(ActivityTypes.ACTIVITY_AWAKE)
			objUnit.NotifyEntity(gc.getInfoTypeForString("MISSION_WAKE"))
			return
		
		# Get the experience needed for the unit
		iExperienceNeeded = objUnit.experienceNeeded()

		iUnitLevel = objUnit.getLevel()
		iFortifyCount = sdGetVal("Military Bases", self.getUniqueUnitID(objUnit), "FortifyCount")
		iFortifyTarget = sdGetVal("Military Bases", self.getUniqueUnitID(objUnit), "FortifyTarget")
				
		# If the unit is ready to be given experience points then
		if(iFortifyTarget <= iFortifyCount):
			# Reset the script data
			sdSetVal("Military Bases", self.getUniqueUnitID(objUnit), "FortifyTarget", self.getNextFortifyTarget(objUnit))
			sdSetVal("Military Bases", self.getUniqueUnitID(objUnit), "FortifyCount",0)

			# Give the unit its experience point(s)
			if(g_bMaxLevelUnitTrainedCapped):
				objUnit.setExperience(objUnit.getExperience()+self.getUnitExperiencePoints(),iExperienceNeeded)
			else:
				objUnit.setExperience(objUnit.getExperience()+self.getUnitExperiencePoints(),objUnit.getExperience()+self.getUnitExperiencePoints())

		if(iFortifyTarget > iFortifyCount):
			sdSetVal("Military Bases", self.getUniqueUnitID(objUnit), "FortifyCount",iFortifyCount+1)
		
	
	# Setup the unit to start training it		
	def startTrainingUnit(self, objUnit):

		# Return immediately if the unit passed in is invalid
		if(objUnit == None):
			return
		
		# Return immediately if the unit passed in is invalid
		if(objUnit.isNone()):
			return 

		# Return immediately if the unit cannot be trained
		if(not self.canTrainUnit(objUnit)):

			# Wipe out the units script data if it is being tracked
			if(sdEntityExists("Military Bases",self.getUniqueUnitID(objUnit))):
				sdEntityWipe("Military Bases", self.getUniqueUnitID(objUnit))	
			return

		# Reset the script data if it was previously set
		if(sdEntityExists("Military Bases",self.getUniqueUnitID(objUnit))):
			sdSetVal("Military Bases", self.getUniqueUnitID(objUnit), "FortifyTarget", self.getNextFortifyTarget(objUnit))
			sdSetVal("Military Bases", self.getUniqueUnitID(objUnit), "FortifyCount",0)

		else:
			self.setupScript(objUnit)	

		objSelectionGroup = objUnit.getGroup()
		objSelectionGroup.setActivityType(ActivityTypes.ACTIVITY_SLEEP)
		objUnit.NotifyEntity(gc.getInfoTypeForString("MISSION_SLEEP"))
		objUnit.finishMoves()
		CyInterface().clearSelectionList()
		
	# Returns a unique ID comprised of the player ID and the unit ID.
	def getUniqueUnitID(self, objUnit):
	
		# Return INVALID immediately if the unit passed in is invalid
		if(objUnit == None):
			return "INVALID"
		
		# Return INVALID immediately if the unit passed in is invalid
		if(objUnit.isNone()):
			return "INVALID"
			
		# Return the string that is the <playerID>-<unitID>
		return str(objUnit.getOwner())+"-"+str(objUnit.getID())
		
		
	# Setup the script data for the unit passed in
	def setupScript(self, objUnit):

		# Return immediately if the unit passed in is invalid
		if(objUnit == None):
			return
		
		# Return immediately if the unit passed in is invalid
		if(objUnit.isNone()):
			return 
			
		scriptDict = { 	"FortifyCount" : 0,
					"FortifyTarget" : self.getNextFortifyTarget(None) }
		sdEntityInit("Military Bases", self.getUniqueUnitID(objUnit), scriptDict)
	
	
	# Returns the list of units in a city.	
	def getCityUnitList(self, objCity):

		# Return an empty list immediately if the city passed in is invalid
		if(objCity == None):
			return []
		
		# Return an empty list immediately if the city passed in is invalid
		if(objCity.isNone()):
			return []
	
		unitList = []
		
		objPlot = objCity.plot()
		
		for i in range(objPlot.getNumUnits()):
			unitList.append(objPlot.getUnit(i))
			
		return unitList
	
	
	# Returns the number units training in a city
	def getNumCityTrainingUnits(self, objCity):
	
		# Return 0 immediately if the city passed in is invalid
		if(objCity == None):
			return []
		
		# Return 0 immediately if the city passed in is invalid
		if(objCity.isNone()):
			return []
	
		# Get the units at the city
		unitList = self.getCityUnitList(objCity)
	
		iTrainingCount = 0
		
		# Go through all of the units in the list
		for objUnit in unitList:

			# Increment the training count if the units are being tracked
			if(sdEntityExists("Military Bases",self.getUniqueUnitID(objUnit))):
				if(objUnit.getGroup().getActivityType() == ActivityTypes.ACTIVITY_SLEEP):
					iTrainingCount += 1
				else:
					sdEntityWipe("Military Bases", self.getUniqueUnitID(objUnit))	
							
		return iTrainingCount
		

	# Return the number of experience points the unit will get
	def getUnitExperiencePoints(self):
	
		iExperiencePoints = g_iBaseTrainingExperience
		
		# If drill sergeants are not required to train units then add their
		# bonus amount to the amount that will be returned
		if(not g_bDrillSergeantsRequiredToTrain):
			iExperiencePoints += g_iDrillSergeantExperienceBonus

		return iExperiencePoints
		
	
	# Returns the next fortify target amount or how many units needs to wait to
	# get its experience points
	def getNextFortifyTarget(self, objUnit):

		# Return 0 immediately if the unit should gain experience every turn
		if(not g_bUnitsGainExperienceRandomly):
			return 0
		
		# Return a number between 0-9 immediately if the unit passed in is 
		# invalid
		if(objUnit == None):
			return gc.getGame().getMapRand().get(5, "Military Bases")
		
		# Return a number between 0-9 immediately if the unit passed in is 
		# invalid
		if(objUnit.isNone()):
			return gc.getGame().getMapRand().get(5, "Military Bases")
			
		iFortifyCount = sdGetVal("Military Bases", self.getUniqueUnitID(objUnit), "FortifyCount")

		iUnitLevel = objUnit.getLevel()

		# Get a random number, anywhere from 0 to the value of fortifyCount
		iRandVal = gc.getGame().getMapRand().get(iFortifyCount, "Military Bases")

		iGameSpeed = gc.getGame().getGameSpeedType()

		if(iGameSpeed < 0):
			iGameSpeed = 1

		iFortifyTarget = ((iUnitLevel + iRandVal) * iGameSpeed)

		return iFortifyTarget		