#
# Random Great Person Births Mod
# CvRandomGreatPersonBirthsEventManager
# 

from CvPythonExtensions import *
import CvUtil

import CvEventManager
import sys
import PyHelpers
import CvConfigParser

gc = CyGlobalContext()	

PyPlayer = PyHelpers.PyPlayer
PyGame = PyHelpers.PyGame()

# Change this value to true if any of the great people should be considered for 
# random generation. If set to false then only great people will be generated 
# from the points generated from the player cities, i.e. if a city is only 
# generating points for a great artist and the value is set to false then only 
# great artists will be randomly generated.
# Default value is false.
g_bAnyRandomGreatPerson = False

# Change this value to increase or decrease the chance that great persons will 
# be born. For instance if a city has 25 great person points, their great 
# person threshold is 100 points and the value is set to 2.0 then the chance 
# that a great person will be generated will be (25*2.0)/100 or 50%. Another
# example is if the value is set to 0.2 instead then the chance that a great
# person will be generated will be (25*0.2)/100 or 5%.
# Default value is 1.0
g_iRandomGreatPersonBirthModifier = 1.0

# Change this value to true if the random great person generated should be the
# the great person with the most points being generated in the city. When set
# to false then a random great person will be generated in a city if it has
# one or more points being generated for it.
# Default value is false.
g_bBestCityGreatPerson = False

# Change this value to true if the random great person born should be treated
# as a bonus and should not increase the great person point threshold.
# Default value is false
g_bBonusGreatPersonBirth = false

# Change this value to true if the regular great person generation should be
# disabled. When set to true cities will not accumilate great person points
# like normal.
# Default value is false
g_bDisableRegularGreatPersonGeneration = false

# Change the value to enable or disable the features from the Random Great 
# Person Births Mod.
# Default value is true
g_bRandomGreatPersonBirthFeaturesEnabled = true

g_greatPeopleList = []


def loadConfigurationData():
	global g_bAnyRandomGreatPerson
	global g_iRandomGreatPersonBirthModifier

	global g_bRandomGreatPersonBirthFeaturesEnabled

	config = CvConfigParser.CvConfigParser("Great Options Mod Config.ini")

	if(config != None):

		g_bRandomGreatPersonBirthFeaturesEnabled = config.getboolean("Great Options Mod", "Random Great Person Birth Features Enabled", True)		

		
	config = CvConfigParser.CvConfigParser("Random Great Person Births Mod Config.ini")

	if(config != None):

		g_bAnyRandomGreatPerson = config.getboolean("Random Great Person Births Mod", "Any Random Great Person", false)

		g_iRandomGreatPersonBirthModifier = config.getfloat("Random Great Person Births Mod", "Random Great Person Birth Modifier", 1.0)

		g_bBestCityGreatPerson = config.getboolean("Random Great Person Births Mod", "Best City Great Person", false)

		g_bBonusGreatPersonBirth = config.getboolean("Random Great Person Births Mod", "Bonus Great Person Birth", false)

		g_bDisableRegularGreatPersonGeneration = config.getboolean("Random Great Person Births Mod", "Disable Regular Great Person Generation", false)


# globals
###################################################
class CvRandomGreatPersonBirthsEventManager:
	def __init__(self, eventManager):
		# initialize base class

		eventManager.addEventHandler("cityDoTurn", self.onCityDoTurn)

		loadConfigurationData()

		self.initGreatPeopleList()
		
			
	def onCityDoTurn(self, argsList):
		'City Production'
		objCity = argsList[0]
		iPlayer = argsList[1]
		
		if(not g_bRandomGreatPersonBirthFeaturesEnabled):
			return
			
		# Return immediately if the cities current great people rate is 0
		if(objCity.getGreatPeopleRate() == 0):
			return
		
		# initialize the great people list if it isn't initialized
		if(len(g_greatPeopleList) == 0):
			self.initGreatPeopleList()
		
		# Get the random chance that a great person will be born
		iChanceGenerateGreatPerson = gc.getGame().getMapRand().get(gc.getPlayer( iPlayer).greatPeopleThreshold()+1,"Great Person Chance")


		# Calculate the test that a great person will be born
		iGenerateGreatPersonTest = g_iRandomGreatPersonBirthModifier * objCity.getGreatPeopleRate()
		
		# Remove any of the great person points if the regular way to generate
		# great persons has been disabled.
		if(g_bDisableRegularGreatPersonGeneration):
			objCity.changeGreatPeopleProgress(-1*objCity.getGreatPeopleRate())
		
		if(iChanceGenerateGreatPerson <= iGenerateGreatPersonTest):
			self.spawnGreatPerson(objCity)

		strTemp = "%s chance %s %s/%s in %s" %(gc.getPlayer( iPlayer).getName(), iChanceGenerateGreatPerson, iGenerateGreatPersonTest, gc.getPlayer( iPlayer).greatPeopleThreshold(), objCity.getName())
		CvUtil.pyPrint(strTemp)
		

	# Creates a great person in the city passed in
	def spawnGreatPerson(self, objCity):
	
		# Return immediately if the city passed in is invalid
		if(objCity == None):
			return
			
		# Return immediately if the city passed in is invalid			
		if(objCity.isNone()):
			return
			
		# If we are picking a random person then get them from the great people 
		# list
		if(g_bAnyRandomGreatPerson):
			iGreatPersonType = g_greatPeopleList[gc.getGame().getMapRand().get(len(g_greatPeopleList), "Random Great Person")]
		# Get the best great person in the city
		elif(g_bBestCityGreatPerson):
			iGreatPersonType = self.getBestGreatPerson(objCity)
		else:
			iGreatPersonType = self.getRandomCityGreatPerson(objCity)

		objCity.createGreatPeople(iGreatPersonType,not g_bBonusGreatPersonBirth)
		if(not g_bBonusGreatPersonBirth):
			objCity.changeGreatPeopleProgress(-1*objCity.getGreatPeopleProgress())


	# Returns a random unit type ID from the list of great persons that have at 
	# least one point being generated in the city passed in.
	def getRandomCityGreatPerson(self, objCity):			

		# initialize the great people list if it isn't initialized
		if(len(g_greatPeopleList) == 0):
			self.initGreatPeopleList()

		# Return any great person immediately if the city passed in is invalid
		if(objCity == None):
			return g_greatPeopleList[gc.getGame().getMapRand().get(len(g_greatPeopleList), "Random Great Person")]
			
		# Return any great person immediately if the city passed in is invalid
		if(objCity.isNone()):
			return g_greatPeopleList[gc.getGame().getMapRand().get(len(g_greatPeopleList), "Random Great Person")]

		cityGreatPeopleList = []
		
		# Go through all of the great people in the game		
		for iUnitType in g_greatPeopleList:

			# If the current great person is better than the current great 
			# person then set it as the best great person
			if(objCity.getGreatPeopleUnitProgress(iUnitType) > 0):
				cityGreatPeopleList.append(iUnitType)

		return cityGreatPeopleList[gc.getGame().getMapRand().get(len(cityGreatPeopleList), "Random Great Person")]


	# Returns the unit type ID for the great person with the highest percentage 
	# in the city passed in.	
	def getBestGreatPerson(self, objCity):			

		# initialize the great people list if it isn't initialized
		if(len(g_greatPeopleList) == 0):
			self.initGreatPeopleList()

		# Return any great person immediately if the city passed in is invalid
		if(objCity == None):
			return g_greatPeopleList[gc.getGame().getMapRand().get(len(g_greatPeopleList), "Random Great Person")]
			
		# Return any great person immediately if the city passed in is invalid
		if(objCity.isNone()):
			return g_greatPeopleList[gc.getGame().getMapRand().get(len(g_greatPeopleList), "Random Great Person")]

		iBestGreatPersonType = g_greatPeopleList[0]

		# Go through all of the great people in the game		
		for iUnitType in g_greatPeopleList:

			# If the current great person is better than the current great 
			# person then set it as the best great person
			if(objCity.getGreatPeopleUnitProgress(iUnitType) > objCity.getGreatPeopleUnitProgress(iBestGreatPersonType)):
				iBestGreatPersonType = iUnitType
				
		return iBestGreatPersonType

	
	# Initializes the great people list with all of the great persons in the 
	# game.
	def initGreatPeopleList(self):
		global g_greatPeopleList
		
		# Go through all of the specialists in the game
		for i in range( gc.getNumSpecialistInfos() ):
			objSpecialistInfo = gc.getSpecialistInfo(i)
			objUnitClassInfo = gc.getUnitClassInfo(objSpecialistInfo.getGreatPeopleUnitClass())
			
			# Continue if unit class info is none
			if(objUnitClassInfo == None):
				continue
				
			iUnitType = objUnitClassInfo.getDefaultUnitIndex()		
		
			g_greatPeopleList.append(iUnitType)
			
			