## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
import CvUtil
import CvScreenEnums
import CvConfigParser
import string

from CvPythonExtensions import *

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

# globals
g_bShowQuestBody = 0
g_bChooseOpen = 0
g_iStartTime = 0

# Change the value to change which type of domestic advisor should be used in 
# the game. 
# The current valid values are:
#	- Vanilla Domestic Advisor
#	- Customizable Domestic Advisor
#   - Modified Special Domestic Advisor
# If an invalid value is used then the value will default to Vanilla Domestic 
# Advisor.
# Default value is Vanilla Domestic Advisor
g_strDomesticAdvisorType = "Vanilla Domestic Advisor"

# Change the value to change which type of foreign advisor should be used in 
# the game. 
# The current valid values are:
#	- Vanilla
#	- Enhanced Foreign Advisor
#	- Exotic Foreign Advisor
# If an invalid value is used then the value will default to Vanilla Foreign 
# Advisor.
# Default value is Vanilla Foreign Advisor
g_strForeignAdvisorType = "Vanilla Foreign Advisor"

# Change the value to change which type of military advisor should be used in 
# the game. 
# The current valid values are:
#	- Vanilla Military Advisor
#	- Enhanced Military Advisor
# If an invalid value is used then the value will default to Vanilla Military  
# Advisor.
# Default value is Vanilla Military Advisor
g_strMilitaryAdvisorType = "Vanilla Military Advisor"

# Change the value to change which type of technology splash screen should 
# be used in the game. 
# The current valid values are:
#	- Vanilla Tech Splash Screen
#	- Enhanced Tech Splash Screen
# If an invalid value is used then the value will default to Vanilla Tech 
# Splash Screen
# Default value is Vanilla Tech Splash Screen
g_strTechSplashScreenType = "Vanilla Tech Splash Screen"

# Change the value to enable or disable the features from the Dead Civ 
# Scoreboard Mod.
# Default value is true
g_bDeadCivScoreboardFeaturesEnabled = false

# Change the value to enable or disable the features from the GP Trickle Mod.
# Default value is true
g_bGPTrickleFeaturesEnabled = true

# Change the value to enable or disable the features from the Unit Allegiance 
# Mod.
# Default value is true
g_bUnitAllegianceFeaturesEnabled = true

# Change the value to enable or disable the features from the Random Great 
# Person Births Mod.
# Default value is true
g_bRandomGreatPersonBirthFeaturesEnabled = true

# Change the value to enable or disable the features from the M.A.D. Nukes Mod.
# Default value is true
g_bMADNukesFeaturesEnabled = true

# Change the value to enable or disable the features from the Not Just Another 
# Game Clock Features Mod.
# Default value is true
g_bNotJustAnotherGameClockFeaturesEnabled = true

# Change the value to enable or disable the features from the No Name Great 
# Person Renamer Mod.
# Default value is true
g_bNoNameGreatPersonRenamerFeaturesEnabled = true

# Change the value to enable or disable the features from the Cultural 
# Influences mod.
# Default value is true
g_bCulturalInfluencesFeaturesEnabled = true

# Change the value to enable or disable the features from the Mercenaries Mod.
# Default value is true
g_bMercenariesFeaturesEnabled = true

# Change the value to enable or disable the features from the Enhanced Culture 
# Conquest mod.
# Default value is true
g_bEnhancedCultureConquestFeaturesEnabled = true

# Change the value to enable or disable the features from the Enhanced Tech 
# Conquest mod.
# Default value is true
g_bEnhancedTechConquestFeaturesEnabled = true

# Change the value to enable or disable the features from the Localized 
# Starting Techs Mod.
# Default value is true
g_bLocalizedStartingTechsFeaturesEnabled = False

# Change the value to enable or disable the features from the Immigration Mod.
# Default value is true
g_bImmigrationModFeaturesEnabled = true

CONFIGURATION_FILE = "Configuration File"
CONFIGURATION_FILES = "Configuration Files"
SECTION_HEAD = "Section Head"
MOD_DESCRIPTION = "Mod Description"
MOD_ENABLED = "Mod Enabled"
GREAT_OPTIONS_CONFIGURATION = "Great Options Configuration"

VANILLA_DOMESTIC_ADVISOR = "Vanilla Domestic Advisor"
CUSTOMIZABLE_DOMESTIC_ADVISOR = "Customizable Domestic Advisor"
MODIFIED_SPECIAL_DOMESTIC_ADVISOR = "Modified Special Domestic Advisor"
HOMEGROWN_DOMESTIC_ADVISOR = "Homegrown Domestic Advisor"

g_DomesticAdvisorMapping = { VANILLA_DOMESTIC_ADVISOR : VANILLA_DOMESTIC_ADVISOR, 
							CUSTOMIZABLE_DOMESTIC_ADVISOR : CUSTOMIZABLE_DOMESTIC_ADVISOR,
							MODIFIED_SPECIAL_DOMESTIC_ADVISOR : MODIFIED_SPECIAL_DOMESTIC_ADVISOR,
							HOMEGROWN_DOMESTIC_ADVISOR : HOMEGROWN_DOMESTIC_ADVISOR } 

VANILLA_FOREIGN_ADVISOR = "Vanilla Foreign Advisor"
ENHANCED_FOREIGN_ADVISOR = "Enhanced Foreign Advisor"
EXOTIC_FOREIGN_ADVISOR = "Exotic Foreign Advisor"

g_ForeignAdvisorMapping = { VANILLA_FOREIGN_ADVISOR : VANILLA_FOREIGN_ADVISOR, 
							ENHANCED_FOREIGN_ADVISOR : ENHANCED_FOREIGN_ADVISOR, 
							EXOTIC_FOREIGN_ADVISOR : EXOTIC_FOREIGN_ADVISOR } 

VANILLA_MILITARY_ADVISOR = "Vanilla Military Advisor"
ENHANCED_MILITARY_ADVISOR = "Enhanced Military Advisor"

g_MilitaryAdvisorMapping = { VANILLA_MILITARY_ADVISOR : VANILLA_MILITARY_ADVISOR, 
							ENHANCED_MILITARY_ADVISOR : ENHANCED_MILITARY_ADVISOR } 


VANILLA_TECH_SPLASH_SCREEN = "Vanilla Tech Splash Screen"
ENHANCED_TECH_SPLASH_SCREEN = "Enhanced Tech Splash Screen"

g_TechSplashScreenMapping = { VANILLA_TECH_SPLASH_SCREEN : VANILLA_TECH_SPLASH_SCREEN, 
							ENHANCED_TECH_SPLASH_SCREEN : ENHANCED_TECH_SPLASH_SCREEN} 

g_ModInfoDict = {}
g_ModConfigurationFileList = []

def loadConfigurationData():
	
	CvUtil.pyPrint("Loading configuration values")

	global g_ModInfoDict
		
	global g_strDomesticAdvisorType
	global g_strForeignAdvisorType 
	global g_strMilitaryAdvisorType 
	global g_strTechSplashScreenType

	global g_bDeadCivScoreboardFeaturesEnabled
	global g_bGPTrickleFeaturesEnabled
	global g_bUnitAllegianceFeaturesEnabled
	global g_bRandomGreatPersonBirthFeaturesEnabled
	global g_bMADNukesFeaturesEnabled
	global g_bNotJustAnotherGameClockFeaturesEnabled
	global g_bNoNameGreatPersonRenamerFeaturesEnabled
	global g_bCulturalInfluencesFeaturesEnabled
	global g_bMercenariesFeaturesEnabled
	global g_bEnhancedCultureConquestFeaturesEnabled 
	global g_bEnhancedTechConquestFeaturesEnabled 
	global g_bLocalizedStartingTechsFeaturesEnabled
	global g_bImmigrationModFeaturesEnabled
	
	config = CvConfigParser.CvConfigParser("Great Options Mod Config.ini")

	if(config != None):
		g_strDomesticAdvisorType = config.get("Great Options Mod", "Domestic Advisor Type", "Vanilla Domestic Advisor")
	
		g_strForeignAdvisorType = config.get("Great Options Mod", "Foreign Advisor Type", "Vanilla Foreign Advisor")

		g_strMilitaryAdvisorType = config.get("Great Options Mod", "Military Advisor Type", "Vanilla Military Advisor")
		
		g_strTechSplashScreenType = config.get("Great Options Mod", "Tech Splash Screen Type", "Vanilla Tech Splash Screen")

		g_bDeadCivScoreboardFeaturesEnabled = config.getboolean("Great Options Mod", "Dead Civ Scoreboard Features Enabled", True)

		g_bGPTrickleFeaturesEnabled = config.getboolean("Great Options Mod", "GP Trickle Features Enabled", True)

		g_bUnitAllegianceFeaturesEnabled = config.getboolean("Great Options Mod", "Unit Allegiance Features Enabled", True)

		g_bRandomGreatPersonBirthFeaturesEnabled = config.getboolean("Great Options Mod", "Random Great Person Birth Features Enabled", True)		

		g_bMADNukesFeaturesEnabled = config.getboolean("Great Options Mod", "MAD Nukes Features Enabled", True)
		
		g_bNotJustAnotherGameClockFeaturesEnabled = config.getboolean("Great Options Mod", "Not Just Another Game Clock Features Enabled", True)
		
		g_bNoNameGreatPersonRenamerFeaturesEnabled = config.getboolean("Great Options Mod", "No Name Great Person Renamer Features Enabled", True)

		g_bCulturalInfluencesFeaturesEnabled = config.getboolean("Great Options Mod", "Cultural Influences Features Enabled", True)

		g_bMercenariesFeaturesEnabled = config.getboolean("Great Options Mod", "Mercenaries Features Enabled", True)

		g_bEnhancedCultureConquestFeaturesEnabled = config.getboolean("Great Options Mod", "Enhanced Culture Conquest Features Enabled", True)

		g_bEnhancedTechConquestFeaturesEnabled = config.getboolean("Great Options Mod", "Enhanced Tech Conquest Features Enabled", True)

		g_bLocalizedStartingTechsFeaturesEnabled = config.getboolean("Great Options Mod", "Localized Starting Techs Features Enabled", True)
		
		g_bImmigrationModFeaturesEnabled = config.getboolean("Great Options Mod", "Immigration Mod Features Enabled", True)

	if(not g_strDomesticAdvisorType in g_DomesticAdvisorMapping.keys()):
		g_strDomesticAdvisorType = "Vanilla Domestic Advisor"

	if(not g_strForeignAdvisorType in g_ForeignAdvisorMapping.keys()):
		g_strForeignAdvisorType = "Vanilla Foreign Advisor"
		
	if(not g_strMilitaryAdvisorType in g_MilitaryAdvisorMapping.keys()):
		g_strMilitaryAdvisorType = "Vanilla Military Advisor"

	if(not g_strTechSplashScreenType in g_TechSplashScreenMapping.keys()):
		g_strTechSplashScreenType = "Vanilla Tech Splash Screen"

	g_ModInfoDict = {
		"Cultural Influences" : {						
			MOD_ENABLED : g_bCulturalInfluencesFeaturesEnabled,
			SECTION_HEAD : ["Cultural Influences"],
			CONFIGURATION_FILE : "Cultural Influences Config.ini",
			MOD_DESCRIPTION : "Cultural Influences Features Enabled" },

		"Dead Civ Scoreboard Mod" : {						
			MOD_ENABLED : g_bDeadCivScoreboardFeaturesEnabled,
			SECTION_HEAD : ["Dead Civ Scoreboard Mod"],
			CONFIGURATION_FILE : "Dead Civ Scoreboard Mod Config.ini",
			MOD_DESCRIPTION : "Dead Civ Scoreboard Features Enabled" },

		"Enhanced Culture Conquest" : {						
			MOD_ENABLED : g_bEnhancedCultureConquestFeaturesEnabled,
			SECTION_HEAD : ["Enhanced Culture Conquest"],
			CONFIGURATION_FILE : "Enhanced Culture Conquest Config.ini",
			MOD_DESCRIPTION : "Enhanced Culture Conquest Features Enabled" },

		"Enhanced Foreign Advisor" : {						
			MOD_ENABLED : true,
			SECTION_HEAD : ["Enhanced Foreign Advisor"],
			CONFIGURATION_FILE : "Enhanced Foreign Advisor Config.ini",
			MOD_DESCRIPTION : "Enhanced Foreign Advisor" },

		"Enhanced Military Advisor" : {						
			MOD_ENABLED : true,
			SECTION_HEAD : None,
			CONFIGURATION_FILE : None,
			MOD_DESCRIPTION : "Enhanced Military Advisor" },

		"Enhanced Tech Conquest" : {						
			MOD_ENABLED : g_bEnhancedTechConquestFeaturesEnabled,
			SECTION_HEAD : ["Enhanced Tech Conquest"],
			CONFIGURATION_FILE : "Enhanced Tech Conquest Config.ini",
			MOD_DESCRIPTION : "Enhanced Tech Conquest Features Enabled" },

		"Enhanced Tech Screen" : {						
			MOD_ENABLED : true,
			SECTION_HEAD : None,
			CONFIGURATION_FILE : None,
			MOD_DESCRIPTION : "Enhanced Tech Screen" },

		"GP Trickle Mod" : {						
			MOD_ENABLED : g_bGPTrickleFeaturesEnabled,
			SECTION_HEAD : ["GP Trickle Mod"],
			CONFIGURATION_FILE : "GP Trickle Mod Config.ini",
			MOD_DESCRIPTION : "GP Trickle Features Enabled" },
			
		"Great Doctor" : {						
			MOD_ENABLED : true,
			SECTION_HEAD : ["Great Doctor"],
			CONFIGURATION_FILE : "Great Doctor Config.ini",
			MOD_DESCRIPTION : "Great Doctor Features Enabled" },
		
		"Great General Amped" : {						
			MOD_ENABLED : true,
			SECTION_HEAD : ["Great General Amped"],
			CONFIGURATION_FILE : "Great General Amped Config.ini",
			MOD_DESCRIPTION : "Great General Amped Features Enabled" },
		
		"Great Statesman" : {						
			MOD_ENABLED : true,
			SECTION_HEAD : ["Great Statesman"],
			CONFIGURATION_FILE : "Great Statesman Config.ini",
			MOD_DESCRIPTION : "Great Statesman Features Enabled" },

		"Immigration Mod" : {						
			MOD_ENABLED : g_bImmigrationModFeaturesEnabled,
			SECTION_HEAD : ["Immigration Mod"],
			CONFIGURATION_FILE : "Immigration Mod Config.ini",
			MOD_DESCRIPTION : "Immigration Mod Features Enabled" },

		"Localized Starting Techs Mod" : {						
			MOD_ENABLED : g_bLocalizedStartingTechsFeaturesEnabled,
			SECTION_HEAD : ["Localized Starting Techs Mod","Plot Type Technology Mappings", "Terrain Type Technology Mappings", "Feature Type Technology Mappings", "Bonus Type Technology Mappings", "Leader Trait Technology Mappings"],
			CONFIGURATION_FILE : "Localized Starting Techs Mod Config.ini",
			MOD_DESCRIPTION : "Localized Starting Techs Features Enabled" },

		"MAD Nukes Mod" : {						
			MOD_ENABLED : g_bMADNukesFeaturesEnabled,
			SECTION_HEAD : ["MAD Nukes Mod"],
			CONFIGURATION_FILE : "MAD Nukes Mod Config.ini",
			MOD_DESCRIPTION : "MAD Nukes Features Enabled" },
			
		"Mercenaries Mod" : {						
			MOD_ENABLED : g_bMercenariesFeaturesEnabled,
			SECTION_HEAD : ["Mercenaries Mod"],
			CONFIGURATION_FILE : "Mercenaries Mod Config.ini",
			MOD_DESCRIPTION : "Mercenaries Features Enabled" },
			
		"Military Bases" : {						
			MOD_ENABLED : true,
			SECTION_HEAD : ["Military Bases"],
			CONFIGURATION_FILE : "Military Bases Config.ini",
			MOD_DESCRIPTION : "Military Bases Features Enabled" },
		
		"No Name Great Person Renamer Mod" : {						
			MOD_ENABLED : g_bNoNameGreatPersonRenamerFeaturesEnabled,
			SECTION_HEAD : ["No Name Great Person Renamer Mod"],
			CONFIGURATION_FILE : "No Name Great Person Renamer Mod Config.ini",
			MOD_DESCRIPTION : "No Name Great Person Renamer Features Enabled" },

		"Not Just Another Game Clock Mod" : {						
			MOD_ENABLED : g_bNotJustAnotherGameClockFeaturesEnabled,
			SECTION_HEAD : ["Not Just Another Game Clock Mod"],
			CONFIGURATION_FILE : "Not Just Another Game Clock Mod Config.ini",
			MOD_DESCRIPTION : "Not Just Another Game Clock Features Enabled" },
					
		"Random Great Person Births Mod" : {						
			MOD_ENABLED : g_bRandomGreatPersonBirthFeaturesEnabled,
			SECTION_HEAD : ["Random Great Person Births Mod"],
			CONFIGURATION_FILE : "Random Great Person Births Mod Config.ini",
			MOD_DESCRIPTION : "Random Great Person Birth Features Enabled" },
					
		"Sniper Mod" : {						
			MOD_ENABLED : true,
			SECTION_HEAD : ["Sniper Mod"],
			CONFIGURATION_FILE : "Sniper Mod Config.ini",
			MOD_DESCRIPTION : "Sniper Features Enabled" },
					
		"Specialist Stacker" : {						
			MOD_ENABLED : true,
			SECTION_HEAD : ["Specialist Stacker"],
			CONFIGURATION_FILE : "Specialist Stacker Config.ini",
			MOD_DESCRIPTION : "Specialist Stacker Features Enabled" },
		
		"Unit Allegiance Mod" : {						
			MOD_ENABLED : g_bUnitAllegianceFeaturesEnabled,
			SECTION_HEAD : ["Unit Allegiance Mod"],
			CONFIGURATION_FILE : "Unit Allegiance Mod Config.ini",
			MOD_DESCRIPTION : "Unit Allegiance Features Enabled" },
			
		}			
		
class CvGreatOptionsConfigInfoScreen:
	
	def __init__(self, iScreenId):
	
		self.iScreenId = iScreenId
		self.SCREEN_NAME = "GreatOptionsConfigInfo"
		self.nWidgetCount = 0
		self.nLineCount = 0
		self.WIDGET_ID = "GreatOptionsConfigInfoWidget"
		self.EXIT_TEXT = None
		self.TITLE = "Title"
		self.strCurrentConfigurationScreen = GREAT_OPTIONS_CONFIGURATION
		
	
	def populateConfigurationlist(self, screen):
	
		global g_ModConfigurationFileList

		g_ModConfigurationFileList = []
		
		listKeys = g_ModInfoDict.keys()
		listKeys.sort()
		screen.addPullDownString(CONFIGURATION_FILES, GREAT_OPTIONS_CONFIGURATION, 0, 0, true)
		
		g_ModConfigurationFileList.append(GREAT_OPTIONS_CONFIGURATION)

		for modInfoKey in listKeys:
			modInfo = g_ModInfoDict[modInfoKey]
			
			if(modInfo[CONFIGURATION_FILE] == None):
				continue
				
			bActive = false
			if(self.strCurrentConfigurationScreen == modInfoKey):
				bActive = true

			screen.addPullDownString(CONFIGURATION_FILES, modInfoKey+" Configuration", len(g_ModConfigurationFileList), len(g_ModConfigurationFileList), bActive)

			g_ModConfigurationFileList.append(modInfoKey)
					
	# Returns the instance of the great options config info screen.						
	def getScreen(self):
		return CyGInterfaceScreen(self.SCREEN_NAME, self.iScreenId)
		
	# Gets the instance of the great options config info screen and hides it.
	def hideScreen(self):
		screen = self.getScreen()
		screen.hideScreen()
		
	# Returns true if the screen is active, false otherwise.	
	def isActive(self):
		return self.getScreen().isActive()
						
	def interfaceScreen(self):
		global g_bShowQuestBody
		global g_iStartTime
		
		# Create a new screen
		screen = self.getScreen()
				
		if screen.isActive():
			return
			
		loadConfigurationData()
		
 		screen.setRenderInterfaceOnly(false);

		iWidth = screen.getXResolution() - 100
		iHeight = screen.getYResolution() - 200
		iX = (screen.getXResolution() - iWidth)/2
		iY = 50
		
		iEXIT_X = iWidth - 40
		iEXIT_Y = iHeight
		
		iCONFIGURATION_TEXT_X = iX + 40
		iCONFIGURATION_TEXT_Y = iEXIT_Y
		
		iCONFIGURATION_LIST_X = iCONFIGURATION_TEXT_X + 200
		iCONFIGURATION_LIST_Y = iCONFIGURATION_TEXT_Y
		
		self.EXIT_TEXT = u"<font=4>" + localText.getText("TXT_KEY_PEDIA_SCREEN_EXIT", ()).upper() + u"</font>"
		CONFIGURATION_DATA = u"<font=4>Configuration Data</font>"
		
		screen.addPanel("MainGreatOptionsConfigPanel", "", "", True, True, iX, iY, iWidth, iHeight, PanelStyles.PANEL_STYLE_BLUELARGE)
		screen.addPanel("InfoPanel", "", "", True, True, iX+40, iY+60, iWidth-80, iHeight-120, PanelStyles.PANEL_STYLE_EMPTY)
	
		screenTitle = u"<font=4b>Great Options Config Info</font>"
		
		screen.setLabel(self.TITLE, "", screenTitle, CvUtil.FONT_CENTER_JUSTIFY, ((iWidth+iX)/2) , iY + 20, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.setLabel(self.getNextWidgetName(), "", CONFIGURATION_DATA, CvUtil.FONT_LEFT_JUSTIFY, iCONFIGURATION_TEXT_X , iCONFIGURATION_TEXT_Y, 0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.addDropDownBoxGFC(CONFIGURATION_FILES, iCONFIGURATION_LIST_X, iCONFIGURATION_LIST_Y, 400, WidgetTypes.WIDGET_GENERAL, -1, -1, FontTypes.GAME_FONT)
	
		self.populateConfigurationlist(screen)
		
		screen.setText(self.getNextWidgetName(), "",self.EXIT_TEXT, CvUtil.FONT_LEFT_JUSTIFY, iEXIT_X, iEXIT_Y, 1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_CLOSE_SCREEN, -1, -1 )

		if(self.strCurrentConfigurationScreen == GREAT_OPTIONS_CONFIGURATION):
			self.drawGreatOptionsTable(screen)
		else:
		
			screenTitle = u"<font=4b>%s Config</font>" %(self.strCurrentConfigurationScreen)
			
			screen.setLabel(self.TITLE, "", screenTitle, CvUtil.FONT_CENTER_JUSTIFY, ((iWidth+iX)/2) , iY + 20, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )

			self.drawConfigurationDataTable(screen)

		screen.setCloseOnEscape(true)
		screen.showWindowBackground(false)
		screen.showScreen(PopupStates.POPUPSTATE_IMMEDIATE, false)

		iWidth = screen.getXResolution()
		iHeight = screen.getYResolution() - 100
		iX = (screen.getXResolution() - iWidth)/2
		iY = 0

		screen.setDimensions(iX, iY, iWidth, iHeight)	


	def drawConfigurationDataTable(self, screen):

		modInfo = g_ModInfoDict[self.strCurrentConfigurationScreen]
		
		screen.attachTableControlGFC("InfoPanel", "SettingsTable", 3, false, false, 32, 32, TableStyles.TABLE_STYLE_STANDARD)
		screen.addTableHeaderGFC("SettingsTable","Option", 0, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.addTableHeaderGFC("SettingsTable","Value", 1, WidgetTypes.WIDGET_GENERAL, -1, -1 )	

		config = CvConfigParser.CvConfigParser(modInfo[CONFIGURATION_FILE])
		configFileSectionHeadList = modInfo[SECTION_HEAD]

		iRowCount = 0

		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=3b>%s</font>" %((string.capwords(modInfo[MOD_DESCRIPTION])+":")), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(modInfo[MOD_ENABLED]==true), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)
		
		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)

		for sectionHead in configFileSectionHeadList:

			configValues = config.items(sectionHead)

			configValues.sort()
			
			if(len(configFileSectionHeadList) > 1):
				iRowCount+= 1
				screen.appendTableRow("SettingsTable")
				screen.setTableText("SettingsTable", 0, iRowCount, u"<font=3b>%s</font>" %(sectionHead), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)

			for tup in configValues:
				iRowCount+= 1
				screen.appendTableRow("SettingsTable")
				screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>%s</font>" %((string.capwords(tup[0])+":")), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)
				
				strSetting = tup[1]
				if(strSetting.count("_") > 0):
					strSetting = strSetting.upper()
				else:
					strSetting = string.capwords(strSetting)
					
				screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(strSetting), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)
			
			if(len(configFileSectionHeadList) > 1):
				iRowCount+= 1
				screen.appendTableRow("SettingsTable")
				screen.setTableText("SettingsTable", 0, iRowCount, u"", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)						

	def drawGreatOptionsTable(self, screen):
		screen.attachTableControlGFC("InfoPanel", "SettingsTable", 3, false, false, 32, 32, TableStyles.TABLE_STYLE_STANDARD)
		screen.addTableHeaderGFC("SettingsTable","Option", 0, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.addTableHeaderGFC("SettingsTable","Value", 1, WidgetTypes.WIDGET_GENERAL, -1, -1 )	
		screen.setTableColumnHeader("SettingsTable", 0, "Option", 375)


		iRowCount = 0
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=3b>Screens:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)						

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Domestic Advisor Type:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_strDomesticAdvisorType), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)						

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Foreign Advisor Type:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_strForeignAdvisorType), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)						

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Military Advisor Type:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_strMilitaryAdvisorType), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)						

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Tech Splash Screen Type:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_strTechSplashScreenType), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)						

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)						

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=3b>Other:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)						

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Cultural Influences Features Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bCulturalInfluencesFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Dead Civ Scoreboard Features Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bDeadCivScoreboardFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)
			
		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Enhanced Culture Conquest Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bEnhancedCultureConquestFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)									

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Enhanced Tech Conquest Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bEnhancedTechConquestFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)									

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>GP Trickle Features Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bGPTrickleFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)									
				
		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Immigration Mod Features Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bImmigrationModFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)								

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Localized Starting Techs Features Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bLocalizedStartingTechsFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)								

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>MAD Nukes Features Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bMADNukesFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)								

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Mercenaries Features Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bMercenariesFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)								

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>No Name Great Person Renamer Features Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bNoNameGreatPersonRenamerFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)						
									
		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Not Just Another Game Clock Features Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bNotJustAnotherGameClockFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)						

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Random Great Person Birth Features Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bRandomGreatPersonBirthFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)						

		iRowCount+= 1
		screen.appendTableRow("SettingsTable")
		screen.setTableText("SettingsTable", 0, iRowCount, u"<font=2b>Unit Allegiance Features Enabled:</font>", "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_RIGHT_JUSTIFY)						
		screen.setTableText("SettingsTable", 1, iRowCount, u"%s" %(g_bUnitAllegianceFeaturesEnabled), "", WidgetTypes.WIDGET_GENERAL, -1,-1, CvUtil.FONT_LEFT_JUSTIFY)	

		
	def update(self, fDelta):

		return
		
	# Will handle the input for this screen...
	def handleInput (self, inputClass):

		if(CONFIGURATION_FILES == inputClass.getFunctionName()):
			
			screen = self.getScreen()
			CvUtil.pyPrint("%s" %(screen.getSelectedPullDownID(CONFIGURATION_FILES)))

			self.hideScreen()

			iIndex = screen.getSelectedPullDownID(CONFIGURATION_FILES)
		
			self.strCurrentConfigurationScreen = g_ModConfigurationFileList[screen.getPullDownData(CONFIGURATION_FILES, iIndex)]
			self.interfaceScreen()
			
		return 0



	# returns a unique ID for a widget in this screen
	def getNextWidgetName(self):
		szName = self.WIDGET_ID + str(self.nWidgetCount + self.iScreenId)
		self.nWidgetCount += 1
		return szName