## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
from CvPythonExtensions import *
import PyHelpers
import CvUtil
import ScreenInput
import CvScreenEnums
import HDomPyCity
import CvDomesticAdvisor

PyPlayer = PyHelpers.PyPlayer
PyCity = HDomPyCity.HDomPyCity

#	IMPORTANT INFORMATION
#	
#	All widget names MUST be unique when creating screens.  If you create
#	a widget named 'Hello', and then try to create another named 'Hello', it
#	will modify the first hello.
#
#	Also, when attaching widgets, 'Background' is a reserve word meant for
#	the background widget.  Do NOT use 'Background' to name any widget, but
#	when attaching to the background, please use the 'Background' keyword.

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

#	Set up the base Coordinates here...
STANDARD_Z = -2.2
TOP_Y = 96
Y_SPACING = 32
DATE_COLUMN = 70
NAME_COLUMN = DATE_COLUMN + 90
CULTURE_COLUMN = NAME_COLUMN + 130
HAPPY_COLUMN = CULTURE_COLUMN + 90
HEALTH_COLUMN = HAPPY_COLUMN + 100
FOOD_COLUMN = HEALTH_COLUMN + 40
PRODUCTION_COLUMN = FOOD_COLUMN + 40
GOLD_COLUMN = PRODUCTION_COLUMN + 40
GREATPEOPLE_COLUMN = GOLD_COLUMN + 40
PRODUCING_COLUMN = GREATPEOPLE_COLUMN + 170

# Sorting information
DATE = 0
NAME = 1
CULTURE = 2
HAPPY = 3
HEALTH = 4
FOOD = 5
PRODUCTION = 6
GOLD = 7
GREATPEOPLE = 8
PRODUCING = 9
POPULATION = 10
UNHAPPY = 11
ANGRY = 12
BAD_HEALTH = 13
CONSUMPTION = 14
SCIENCE = 15
CULTURE_RATE = 16
GREATPEOPLE_RATE = 17

# What are we sorting by?
iSortingBy = DATE
iSortingDown = True

# Listbox identifiers...
UNIT_CHOSEN = 0
BUILDING_CHOSEN = 1
PROJECT_CHOSEN = 2
PROCESS_CHOSEN = 3

class CvHomegrownDomesticAdvisor(CvDomesticAdvisor.CvDomesticAdvisor):
	"Domestic Advisor Screen"
	def __init__(self):
		self.listSelectedCities = []

	# Screen construction function
	def interfaceScreen(self):
	
		# This will get the actual Player class into iPlayer
		iPlayer = PyPlayer(CyGame().getActivePlayer())
		
		# Create a new screen, called DomesticAdvisur, using the file CvDomesticAdvisor.py for input
		screen = CyGInterfaceScreen( "DomesticAdvisor", CvScreenEnums.DOMESTIC_ADVISOR )
		
		################
		## Auto Resize Values ## 
		###############
		# nScreenPosY - the top edge of the Domestic Advisor Screen (DAS)
		# 56 is enough to show all top elements of the UI with a bit of padding
		nScreenPosY = 66
		# nScreenPaddingX - the padding of the DAS from the left and right edges of the screen
		# the DAS is horizontally centered so the padding is the same for both edges
		# this value is also used as the X position of the DAS
		nScreenPadX = 40
		## WARNING:
		## If the horizontal padding results in a window narrower than 1024 pixels (e.g. resolution of 1152 and padding of 100) ,
		## then the last column(s) will be cut-off since the table is build to have a minimum width of 1024.
		# nScreenPadBottom - the distance at which the DAS ends from the bottom of the screen
		# 170 is enough to see all bottom UI elements except for the left tooltip and the right civilizations info
		nScreenPadBottom = 180
		
		# if resolution is 1024 then remove horizontal padding, that way you can have padding at high resolutions,
		# but at the smallest resolution the table is readable and the DAS is properly positioned
		if( screen.getXResolution() == 1024 ):
			nScreenPadX = 0
		
		# set up DAS dimensions
		nWindowWidth = screen.getXResolution() - nScreenPadX * 2
		nWindowHeight = screen.getYResolution() - (nScreenPosY + nScreenPadBottom)
		
		# exit button positioning - aligned to bottom right corner of the DAS
		nExitX = nScreenPadX + nWindowWidth - 28
		nExitY = nScreenPosY + nWindowHeight - 34
		
		# nTablePadBottom - the distance at which the table (inside the DAS) ends from the bottom of the DAS
		# increase this value if you want to put more than just text buttons there
		nTablePadBottom = 40
		# nTablePad - the padding of the table to the DAS on the left, top and right edges; default is 20
		nTablePad = 20
		
		# set up the table inside the DAS
		# these are declared as self.[variable name] so that another function (the one that draws the table) can see them
		self.nTablePosX = nScreenPadX + nTablePad
		self.nTablePosY = nScreenPosY + nTablePad
		self.nTableWidth = nWindowWidth - nTablePad * 2
		self.nTableHeight = nWindowHeight - (nTablePad + nTablePadBottom)
		
		# the two dynamic columns are name and production, production will grow by 2/3 of the window's width increase
		# and name will grow by 1/3 of the window's width increase
		nWidthDelta = nWindowWidth - 1024
		nThirdWidthDelta = nWidthDelta / 3
		self.nNameWidth = 85 + nThirdWidthDelta
		self.nProductWidth = 111 + nWidthDelta - nThirdWidthDelta
		###############
		## Auto Resize Values ##
		################
		
		screen.setDimensions(0, 0, screen.getXResolution(), screen.getYResolution())
		screen.showScreen(PopupStates.POPUPSTATE_IMMEDIATE, False)
	
		# Here we set the background widget and exit button, and we show the screen
		screen.addPanel( "DomesticAdvisorBG", u"", u"", True, False, nScreenPadX, nScreenPosY, nWindowWidth, nWindowHeight, PanelStyles.PANEL_STYLE_MAIN )
		
		# Draw the exit button
		screen.setText("DomesticExit", "Background", localText.getText("TXT_KEY_PEDIA_SCREEN_EXIT", ()).upper(), CvUtil.FONT_RIGHT_JUSTIFY, nExitX, nExitY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_CLOSE_SCREEN, -1, -1 )
	
		# Header...
		#szText = "<font=4>" + localText.getText("TXT_KEY_DOMESTIC_ADVISOR_TITLE", ()).upper() + "</font>"
		#screen.setLabel( "DomesticTitleHeader", "Background", szText, CvUtil.FONT_CENTER_JUSTIFY, 472, 40, STANDARD_Z, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		# Erase the flag?
		CyInterface().setDirty(InterfaceDirtyBits.MiscButtons_DIRTY_BIT, True)

		# Draw the city list...
		self.drawContents( )
		
	# headers...
	def drawHeaders( self ):

		# Get the screen and the player
		screen = CyGInterfaceScreen( "DomesticAdvisor", CvScreenEnums.DOMESTIC_ADVISOR )
		
		# Zoom to City
		screen.setTableColumnHeader( "CityListBackground", 0, "", 30 )
		
		# Date Founded Column
		screen.setTableColumnHeader( "CityListBackground", 1, "<font=2>" + localText.getText("TXT_KEY_DOMESTIC_ADVISOR_FOUNDED", ()).upper() + "</font>", 68 )
		
		# Name Column
		screen.setTableColumnHeader( "CityListBackground", 2, "<font=2>" + localText.getText("TXT_KEY_DOMESTIC_ADVISOR_NAME", ()).upper() + "</font>", self.nNameWidth )
		
		# Population Column
		screen.setTableColumnHeader( "CityListBackground", 3, "<font=2>" + localText.getText("TXT_KEY_POPULATION", ()).upper() + "</font>", 35 )
		
		# Growth Column
                screen.setTableColumnHeader( "CityListBackground", 4, "<font=2>" + (u"%c" % gc.getYieldInfo(YieldTypes.YIELD_FOOD).getChar()) + "</font>", 30 )

                # Happiness Column
		screen.setTableColumnHeader( "CityListBackground", 5, "<font=2>" + (u"%c" % CyGame().getSymbolID(FontSymbols.HAPPY_CHAR)) + "</font>", 30 )
		
		## Unhappiness Column
		#screen.setTableColumnHeader( "CityListBackground", 5, "<font=2>" + (u"%c" % CyGame().getSymbolID(FontSymbols.UNHAPPY_CHAR)) + "</font>", 40 )
		
		## Angry Column
		#screen.setTableColumnHeader( "CityListBackground", 6, "<font=2>" + (u"%c" % CyGame().getSymbolID(FontSymbols.ANGRY_POP_CHAR)) + "</font>", 40 )
		
		## Food Column
		#screen.setTableColumnHeader( "CityListBackground", 7, "<font=2>" + (u"%c" % gc.getYieldInfo(YieldTypes.YIELD_FOOD).getChar()) + "</font>", 40 )
		
		# Health Column
		screen.setTableColumnHeader( "CityListBackground", 6, "<font=2>" + (u"%c" % CyGame().getSymbolID(FontSymbols.HEALTHY_CHAR)) + "</font>", 30 )
		
		## Bad Health Column
		#screen.setTableColumnHeader( "CityListBackground", 9, "<font=2>" + (u"%c" % CyGame().getSymbolID(FontSymbols.UNHEALTHY_CHAR)) + "</font>", 40 )
		
		## Food Consumed Column
		#screen.setTableColumnHeader( "CityListBackground", 10, "<font=2>" + (u"%c" % CyGame().getSymbolID(FontSymbols.EATEN_FOOD_CHAR)) + "</font>", 40 )
		
		# Landmarks Column
                screen.setTableColumnHeader( "CityListBackground", 7, "<font=2>" + localText.getText("TXT_KEY_WB_BUILDINGS", ()) + "</font>", 78 )

                # City Garrison Column
                szText = u"%c" % CyGame().getSymbolID(FontSymbols.STRENGTH_CHAR)
                screen.setTableColumnHeader( "CityListBackground", 8, "<font=2>" + szText + "</font>", 30 )
                               
                # Religions Column
                screen.setTableColumnHeader( "CityListBackground", 9, "<font=2>" + localText.getText("TXT_KEY_ADVISOR_RELIGION", ()) + "</font>", 95 )

                # Production Column
		screen.setTableColumnHeader( "CityListBackground", 10, "<font=2>" + (u"%c" % gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar()) + "</font>", 35 )
		
		# Commerce Column
		screen.setTableColumnHeader( "CityListBackground", 11, "<font=2>" + (u"%c" % gc.getYieldInfo(YieldTypes.YIELD_COMMERCE).getChar()) + "</font>", 35 )
		
		# City Maintenance Column
                szText = u"%c" % CyGame().getSymbolID(FontSymbols.BAD_GOLD_CHAR)
                screen.setTableColumnHeader( "CityListBackground", 12, "<font=2>" + szText + "</font>", 30 )

                # Gold Column
                szText = u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar())
                screen.setTableColumnHeader( "CityListBackground", 13, "<font=2>" + szText + "</font>", 40 )

                # Research Column
		szText = u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_RESEARCH).getChar())
		screen.setTableColumnHeader( "CityListBackground", 14, "<font=2>" + szText + "</font>", 40 )
		
		# Culture per turn Column
		screen.setTableColumnHeader( "CityListBackground", 15, "<font=2>" + (u"%c" % gc.getCommerceInfo(CommerceTypes.COMMERCE_CULTURE).getChar()) + "</font>", 40 )
		
		# Culture total Column
		screen.setTableColumnHeader( "CityListBackground", 16, "<font=2>" + localText.getText("TXT_KEY_TOTAL", ()).upper() + "</font>", 55 )
		
		# Great Person per turn Column
		screen.setTableColumnHeader( "CityListBackground", 17, "<font=2>" + (u"%c" % CyGame().getSymbolID(FontSymbols.GREAT_PEOPLE_CHAR)) + "</font>", 40 )
		
		# Great Person total Column
		screen.setTableColumnHeader( "CityListBackground", 18, "<font=2>" + localText.getText("TXT_KEY_TOTAL", ()).upper() + "</font>", 45 )
		
		# Production Column
		screen.setTableColumnHeader( "CityListBackground", 19, "<font=2>" + localText.getText("TXT_KEY_DOMESTIC_ADVISOR_PRODUCING", ()).upper() + "</font>", self.nProductWidth )
	
	# Function to draw the contents of the cityList passed in
	def drawContents (self):
	
		# Get the screen and the player
		screen = CyGInterfaceScreen( "DomesticAdvisor", CvScreenEnums.DOMESTIC_ADVISOR )
		iPlayer = PyPlayer(CyGame().getActivePlayer())
		
		screen.moveToFront( "Background" )
		
		# Build the table
		# the X/Y pos and the Width/Height avlues are created in the beginning of interfaceScreen function
		screen.addTableControlGFC( "CityListBackground", 20, self.nTablePosX, self.nTablePosY, self.nTableWidth, self.nTableHeight, True, False, 32, 32, TableStyles.TABLE_STYLE_STANDARD )
		screen.enableSelect( "CityListBackground", True )
		screen.enableSort( "CityListBackground" )
		screen.setStyle("CityListBackground", "Table_StandardCiv_Style")

		# Loop through the cities
		cityList = iPlayer.getCityList()
		i = 0
		for pLoopCity in cityList:

			screen.appendTableRow( "CityListBackground" )
	
			if (pLoopCity.getName() in self.listSelectedCities):
				screen.selectRow( "CityListBackground", i, True )

			szWidgetName = "ZoomCity" + str(i)
			screen.setImageButton( szWidgetName, ArtFileMgr.getInterfaceArtInfo("INTERFACE_BUTTONS_CITYSELECTION").getPath(), 0, 0, 24, 24, WidgetTypes.WIDGET_ZOOM_CITY, pLoopCity.getOwner(), pLoopCity.getID() )
			screen.attachControlToTableCell( szWidgetName, "CityListBackground", i, 0 )
			
			# Founded date first...
			szIDText = "Date" + str(pLoopCity.getID())
			
			iTurnTime = pLoopCity.getGameTurnFounded()
			szFounded = ""
			szFounded = szFounded + unicode(CyGameTextMgr().getTimeStr(iTurnTime, false))
			#if pLoopCity.isCapital():
			#	szFounded = szFounded + (u"%c" % CyGame().getSymbolID(FontSymbols.STAR_CHAR))
			
			screen.setTableDate( "CityListBackground", 1, i, szFounded, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

                        # Turns til Growth
                        iGrowth = pLoopCity.getFoodRate() - pLoopCity.foodConsumption(False, 0)
			szGrowth = ""
			if iGrowth == 0 or pLoopCity.GetCy().isFoodProduction():
                                szGrowth += "0"
                                szGrowth = localText.changeTextColor(szGrowth, gc.getInfoTypeForString("COLOR_RED"))
                        if iGrowth > 0 and not pLoopCity.GetCy().isFoodProduction():
                                szGrowth += unicode(pLoopCity.GetCy().getFoodTurnsLeft())
 			if iGrowth < 0:
                                szGrowth += u"%c" % CyGame().getSymbolID(FontSymbols.BAD_FOOD_CHAR)

                        screen.setTableInt( "CityListBackground", 4, i, szGrowth, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

			# Happiness
			szHappiness = ""
                        iHappiness = (pLoopCity.getHappyPopulation() - pLoopCity.getUnhappyPopulation())
                        if iHappiness >= 0:
                                szHappiness += unicode(iHappiness)
                        else:
                                szHappiness += u"%c" % CyGame().getSymbolID(FontSymbols.UNHAPPY_CHAR)

                        screen.setTableInt( "CityListBackground", 5, i, szHappiness, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

                        # Health
                        iHealth = pLoopCity.getGoodHealth() - pLoopCity.getBadHealth()
			szHealth = ""
			if iHealth < 0:
                                szHealth += u"%c" % CyGame().getSymbolID(FontSymbols.UNHEALTHY_CHAR)
                        else:
                                szHealth += unicode(iHealth)

                        screen.setTableInt( "CityListBackground", 6, i, szHealth, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

                        # Landmarks Column
        		szLandmarks = ""
                        if pLoopCity.GetCy().isGovernmentCenter():
				szStar = u"%c" % CyGame().getSymbolID(FontSymbols.STAR_CHAR)
				szSilverStar = u"%c" % CyGame().getSymbolID(FontSymbols.SILVER_STAR_CHAR)
				if pLoopCity.isCapital():
					szLandmarks += szStar
				else:
					szLandmarks += szSilverStar

                            ### add National Wonder Landmarks
                        iI = 0
                        for iI in range (64,75):   ### hard coded building id strings from xml
                                if pLoopCity.hasBuilding(iI):
					szBullet = u"%c" % CyGame().getSymbolID(FontSymbols.BULLET_CHAR)
                                        szLandmarks += szBullet
                            ### end National Wonder Landmarks
                        if pLoopCity.GetCy().isPower() and pLoopCity.hasBuilding(17):   ### Factory
				szPower = u"%c" % CyGame().getSymbolID(FontSymbols.POWER_CHAR)
                                szLandmarks += szPower
                        if pLoopCity.GetCy().isDisorder():
				szOccu = u"%c" % CyGame().getSymbolID(FontSymbols.OCCUPATION_CHAR)
				szAngry = u"%c" % CyGame().getSymbolID(FontSymbols.ANGRY_POP_CHAR)
                                if pLoopCity.isOccupation():
                                        szLandmarks += szOccu +":"+unicode(pLoopCity.getOccupationTimer())
                                else:
                                        szLandmarks += szAngry
                            ## We Love the King Day
                        if pLoopCity.getMaintenance() == 0 and pLoopCity.getPopulation() > 7 and not pLoopCity.isDisorder():
                                szWLTKD = u"%c" % CyGame().getSymbolID(FontSymbols.HAPPY_CHAR)
                                szLandmarks += szWLTKD
                            ## Airports and Drydocks
                        if pLoopCity.hasBuilding(14):   ### Drydock
                                szLandmarks += "D "
                        if pLoopCity.hasBuilding(15):   ### Airport
                                szLandmarks += "A "

                        screen.setTableText( "CityListBackground", 7, i, szLandmarks, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
                
                        # Garrison
                        iGarrison = pLoopCity.GetCy().getMilitaryHappinessUnits()
			szGarrison = ""
			szGarrison += unicode(iGarrison)
			if iGarrison < 3:
                                szGarrison = localText.changeTextColor(szGarrison, gc.getInfoTypeForString("COLOR_RED"))

                        screen.setTableInt( "CityListBackground", 8, i, szGarrison, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
                
                        # Religions
                        szReligions = ""
                        lHolyCity = pLoopCity.getHolyCity()
                        if lHolyCity:
                                iI = 0
                                for iI in range(len(lHolyCity)):
                                        szReligions += u"%c" %(gc.getReligionInfo(lHolyCity[iI]).getHolyCityChar())
                        lReligions = pLoopCity.getReligions()
                        if lReligions:
                                iI = 0
                                for iI in range(len(lReligions)):
                                        if lReligions[iI] in lHolyCity:
                                                continue
                                        szReligions += u"%c" %(gc.getReligionInfo(lReligions[iI]).getChar())

                        screen.setTableText( "CityListBackground", 9, i, szReligions, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

                        # maintenance
                        iMaintenance = pLoopCity.getMaintenance()
                        szMaintenance = ""
##                        if iMaintenance == 0:
##                                szMaintenance += u"%c" % CyGame().getSymbolID(FontSymbols.HAPPY_CHAR)
                        szMaintenance += unicode(iMaintenance)

                        screen.setTableInt( "CityListBackground", 12, i, szMaintenance, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

                        # actual gold

                        iGold = pLoopCity.calculateGoldRate() - pLoopCity.getMaintenance()
#                        iGold = pLoopCity.reallyGold() - pLoopCity.GetCy().getMaintenance()
                        szGold = unicode(iGold)
                        if iGold <= 0:
                                szGold = localText.changeTextColor(szGold, gc.getInfoTypeForString("COLOR_RED"))
                        
                        screen.setTableInt( "CityListBackground", 13, i, szGold, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

			self.updateTable(pLoopCity, i)

			i += 1
		
		self.drawHeaders()
		
		screen.moveToBack( "DomesticAdvisorBG" )
		
		self.updateAppropriateCitySelection(len(cityList))


	def updateTable(self, pLoopCity, i):

		screen = CyGInterfaceScreen( "DomesticAdvisor", CvScreenEnums.DOMESTIC_ADVISOR )

		# Date Founded
		#screen.setTableDate( "CityListBackground", 1, i, szFounded, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		# City name...
		screen.setTableText( "CityListBackground", 2, i, pLoopCity.getName(), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
		
		# Population
		screen.setTableInt( "CityListBackground", 3, i, unicode(pLoopCity.getPopulation()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		# Growth...
                #screen.setTableInt( "CityListBackground", 4, i, szGrowth, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

                # Happiness...
		#screen.setTableInt( "CityListBackground", 5, i, szHappiness, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		## Unhappiness...
		#screen.setTableInt( "CityListBackground", 5, i, unicode(pLoopCity.getUnhappyPopulation()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		## Angry...	
		#screen.setTableInt( "CityListBackground", 6, i, unicode(pLoopCity.getAngryPopulation()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		## Food status...
		#screen.setTableInt( "CityListBackground", 7, i, unicode(pLoopCity.getFoodRate()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
		
		# Health...
		#screen.setTableInt( "CityListBackground", 6, i, szHealth, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		## Unhealthy...			
		#screen.setTableInt( "CityListBackground", 9, i, unicode(pLoopCity.getBadHealth()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		## Food consumed
		#screen.setTableInt( "CityListBackground", 10, i, unicode(pLoopCity.foodConsumption( False, 0 )), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		# Landmarks Column
                #screen.setTableText( "CityListBackground", 7, i, szLandmarks, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
                
                # City Garrison Column
                #screen.setTableInt( "CityListBackground", 8, i, szGarrison, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )
                               
                # Religions Column
                #screen.setTableText( "CityListBackground", 9, i, szReligions, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

                # Production status...
		screen.setTableInt( "CityListBackground", 10, i, unicode(pLoopCity.getProductionRate()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		# Commerce status...
		screen.setTableInt( "CityListBackground", 11, i, unicode(pLoopCity.calculateGoldRate()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		# City Maintenance Column
                #screen.setTableInt( "CityListBackground", 12, i, szLandmarks, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

                # Gold Column
                #screen.setTableInt( "CityListBackground", 13, i, szGold, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

                # Science rate...
		screen.setTableInt( "CityListBackground", 14, i, unicode(pLoopCity.getResearchRate()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		# Culture per turn status...
		screen.setTableInt( "CityListBackground", 15, i, unicode(pLoopCity.getCultureCommerce()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		# Total Culture			
		screen.setTableInt( "CityListBackground", 16, i, unicode(pLoopCity.getCulture()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		# Great Person per turn 
		screen.setTableInt( "CityListBackground", 17, i, unicode(pLoopCity.getGreatPeopleRate()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		# Total Great Person
		screen.setTableInt( "CityListBackground", 18, i, unicode(pLoopCity.getGreatPeopleProgress()), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

		# Producing	
		screen.setTableText( "CityListBackground", 19, i, pLoopCity.getProductionName() + " (" + str(pLoopCity.getGeneralProductionTurnsLeft()) + ")", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY )

