#
# Sniper Mod
# CityUtils
# 

from CvPythonExtensions import *

import Popup as PyPopup

import CvUtil

import CvConfigParser
import sys
import PyHelpers
			
gc = CyGlobalContext()	

PyPlayer = PyHelpers.PyPlayer
PyGame = PyHelpers.PyGame()
localText = CyTranslator()

# Change this to true if when sniper units target and kill specialists the
# population amount should also be reduced by one.
# Default value is false.
g_bSniperSpecialistsKillReducesCityPopulation = false

# Set to true to print out debug messages in the logs
g_bDebug = false

LOG_ONLY = 1
SCREEN_ONLY = 2
LOG_AND_SCREEN = 3

# Change to redirect the debug messages to the logs, screen or both.
g_iDebugMessageTarget = LOG_ONLY

# Returns the lists of cities trading with the city passed in that don't 
# belong to the owner of the city passed in.
def getTradeCityList(objCity):
	
	iCounter = 0
	tradeCityList = []

	# Return an empty list immediately if the city passed in is invalid				
	if(objCity == None):
		return []
			
	# Return an empty list immediately if the city passed in is invalid
	if(objCity.isNone()):
		return []
	
	# Get the owner of the city
	iCityOwnerId = objCity.getOwner()
		
	# Go through an endless loop, we'll hopefully break out of it sooner 
	# or later
	while(true):
		objTradeCity = objCity.getTradeCity(iCounter)
		
		# Continue if the owner of the current city is the same as the 
		# owner of the city passed in
		if(iCityOwnerId == objTradeCity.getOwner()):
			iCounter+=1
			continue
			
		# Append the current city to the city list if it is valid
		if (objTradeCity and objTradeCity.getOwner() >= 0):
			tradeCityList.append(objTradeCity)
				
		else:
			break
			
		iCounter+=1
							
	return tradeCityList


			
# Returns the lists of cities trading with the city passed in
def getCompleteTradeCityList(objCity):
	
	iCounter = 0
	tradeCityList = []

	# Return an empty list immediately if the city passed in is invalid				
	if(objCity == None):
		return []
			
	# Return an empty list immediately if the city passed in is invalid
	if(objCity.isNone()):
		return []
	
	# Get the owner of the city
	iCityOwnerId = objCity.getOwner()
		
	# Go through an endless loop, we'll hopefully break out of it sooner 
	# or later
	while(true):
		objTradeCity = objCity.getTradeCity(iCounter)
					
		# Append the current city to the city list if it is valid
		if (objTradeCity and objTradeCity.getOwner() >= 0):
			tradeCityList.append(objTradeCity)
				
		else:
			break
			
		iCounter+=1
							
	return tradeCityList

class CityUtils:

	def __init__(self):
		
		global g_bSniperSpecialistsKillReducesCityPopulation

		config = CvConfigParser.CvConfigParser("Sniper Mod Config.ini")

		if(config != None):
			g_bSniperSpecialistsKillReducesCityPopulation = config.getboolean("Sniper Mod", "Sniper Specialists Kill Reduces City Population", False)


	# Makes all of the citizens working the plots around a city flee into the
	# city.
	def citizensFleeToCity(self, objCity):

		bCitizensFled = false
		
		# Return false immediately if the city passed in is invalid
		if(objCity == None):
			return false

		# Return false immediately if the city passed in is invalid
		if(objCity.isNone()):
			return false
		
		# Go through all the plots the city can extend into.			
		for i in range(1,21):
			# If the plot is being worked then send the citizen into the city
			if(objCity.isWorkingPlotByIndex(i)):
				objCity.alterWorkingPlot(i)
				bCitizensFled = true
		
		return bCitizensFled
		

	# Attempts to remove the target specialist from the city passed in. It will
	# return true if it was able to remove the specialist, false otherwise. 
	# This method WILL return false for any free specialists provided by a 
	# building and specialists automatically allocated by the game.
	def changeCitySpecialistPopulation(self, objCity, iTargetSpecialist):
		' true - if specialist was successfully removed '
		
		# Returns false immediately if the city passed in was invalid
		if(objCity == None):
			return false

		# Return false immediately if the city passed in is invalid
		if(objCity.isNone()):
			return false
		
		# Get the initial specialist count
		iInitialSpecialistCount = objCity.getSpecialistCount(iTargetSpecialist)
		
		if(objCity.getFreeSpecialistCount(iTargetSpecialist) > 0):
			objCity.changeFreeSpecialistCount(iTargetSpecialist, -1)
		elif(objCity.getFreeSpecialistCount(iTargetSpecialist) == 0):
			objCity.alterSpecialistCount(iTargetSpecialist, -1)			
			if(g_bSniperSpecialistsKillReducesCityPopulation):
				objCity.setPopulation(objCity.getPopulation()-1)
		
		if(iInitialSpecialistCount == objCity.getSpecialistCount(iTargetSpecialist)):
			return false
		
		return true
